#include "TCP_Server_BSD_Communicator.h"

/****************************************************************
 * Simple initialization utility function.
 * PRE: queueLength >= 0.
 * POST: myQueueLength == queueLength &&
 *        other data members have received default values.
 ****************************************************************/
void TCP_Server_BSD_Communicator::setQueueLength(int queueLength)
{
  if (queueLength <= 0)
    fatal("TCP_Server_BSD_Communicator(): queueLength must be positive");

  myQueueLength = queueLength;
  myConnectionDescriptor = -1;
}


/*****************************************************************
 * Close the socket's session descriptor.
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/
void TCP_Server_BSD_Communicator::closeConnection()
{
  if (hasConnection())
    {
      std::close(getDescriptor());
      myConnectionDescriptor = -1;
      setConnected(false);
    }
}

// for the parent server in conccurent servers to "forget" 
//  connection info which children are taking care of
//  Basically, this just sets current connection descriptor
//  to -1, and sets the connection status to false.
void TCP_Server_BSD_Communicator::forgetConnection()
{
   if (hasConnection()) 
     {
       myConnectionDescriptor = -1;
       setConnected(false);
     }
}


/****************************************************************
 * Accept a connection.
 * PRE: I am listening for a connection.
 * POST: Someone has connected to me and 
 *        a new socket has been created for their connection and
 *        sessionDescriptor refers to this new socket.
 ****************************************************************/
void TCP_Server_BSD_Communicator::acceptConnection()
{
  if (hasConnection())
    forgetConnection(); 

  struct sockaddr_in remote_address;
  socklen_t sock_size = sizeof(remote_address);

  myConnectionDescriptor = accept(getDescriptor(), 
				(struct sockaddr*)&remote_address,
				&sock_size);

  setClient(SocketAddress(remote_address));
  Server_BSD_Communicator::setConnected(true);
}


/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/
TCP_Server_BSD_Communicator::~TCP_Server_BSD_Communicator()
{
  if (hasConnection())
    closeConnection();    // connection descriptor

  if (isOn())
    close();          // inherited descriptor
}
