#include "Communicator.h"

/***************************************
 * start the Communicator (called by subclasses)
 * PRE: service is the address of a defined Service object.
 * POST: myIsOn == true && myServiceInfo = service &&
 *        mySocketAddress is initialized &&
 *        the endpoint has been allocated.
 ******************************************/

void Communicator::start(Service * service)
{
  if (myIsOn)
    stop();

  setServiceInfo(service);
  allocate();         // This allocates an endpoint. 
		      // polymorphic: socket() in BSD, Winsock;
                      //              t_open() in TLI
  myIsOn = true;      
}


/******************************************************
 * Service mutator
 * PRE: service is a pointer to a valid TCP/IP Service
 * POST: myServiceInfo == service.
 ******************************************************/

void Communicator::setServiceInfo(Service * service)
{
  if (myServiceInfo != NULL)
    delete myServiceInfo;

  myServiceInfo = service;
}


/*****************************************************
 * (Polymorphic) compare() (called by relational ops)
 * PRE: netObj is a Communicator object.
 * POST: RV == 0 iff we are the same Communicator;
 *             1 otherwise.
 *****************************************************/

int Communicator::compare(const NetworkObject & netObj) const
{
  const Communicator * ptr = dynamic_cast<const Communicator*>(&netObj);
  if ((*myServiceInfo) == (*(ptr->serviceInfo())))
    return 0;
  else
    return 1;
}

