#include "Host.h"
/***********************************************
 * Explicit value constructor
 * PRE: name is the name of a valid internet host,
 *       or a valid IP address (in dotted notation).
 * POST: myInfo points to a struct hostent
 *         properly initialized for name.
 ***********************************************/

Host::Host(const string & name)
{
  struct hostent * ptr;

  // use DNS lookup to try and find host info
  if ((ptr = gethostbyname(name.c_str())))
    {
      myName = ptr->h_name;
      memcpy(&myAddress, ptr->h_addr, sizeof(unsigned));
      myAddressString = inet_ntoa(myAddress);
    }
  else
    fatal("Host: host %s not known", name.c_str());

  // if they gave an IP address, h_name needs to be filled in
  if (isdigit(name[0]))
    // try gethostbyaddr() to fill in h_name field
    if ((ptr = gethostbyaddr(ptr->h_addr, sizeof(unsigned), AF_INET)))
      myName = ptr->h_name;
    else
      // warn user, but let execution continue
      warning("Host::Host(name): no host found for address: %s", 
	      name.c_str());
}


Host::Host(const struct in_addr & ipAddress)
{
  struct hostent * ptr;

  if ((ptr = gethostbyaddr((char*) &ipAddress, sizeof(unsigned), AF_INET)))
    {
      myName = ptr->h_name;
      memcpy(&myAddress, ptr->h_addr, sizeof(unsigned));
      myAddressString = inet_ntoa(myAddress);
    }
}
/****************************************************
 * (Polymorphic) compare() method used by relational ops
 * PRE: netObj is a Host.
 * POST: RV == 0 if our addresses are equal;
 *             1 otherwise.
 ****************************************************/

int Host::compare(const NetworkObject & netObj) const
{
  const Host * ptr = dynamic_cast<const Host*>(&netObj);

  if (!ptr)
    fatal("Host::compare(): Unable to compare %s and %s",
	  className().c_str(), netObj.className().c_str());

  return addressNBO() - ptr->addressNBO();
}

