#include "Proto.h"
// uses Proto.h insteadl of protocol.h b/c VC++ had some other file
//   called protocol.h which it liked to use instead.

/***********************************************************
 * Construct Protocol from name.
 * PRE: name is either "udp" or "tcp".
 * POST: myName == name && myNumber is set to its number.
 ***********************************************************/

Protocol::Protocol(const string & name)
{
  struct protoent * ptr = getprotobyname(name.c_str());
  if (ptr)
    {
      myName = name;
      myNumber = ptr->p_proto;
    }
  else
    fatal("\nProtocol: protocol %s not recognized.\n", name.c_str());
}


/***********************************************************
 * Construct Protocol from number
 * PRE: number is a recognized protocol number.
 * POST: myName == its name && myNumber == number.
 ***********************************************************/

Protocol::Protocol(const unsigned short number)
{
  struct protoent * ptr = getprotobynumber(number);
  if (ptr)
    {
      myName = ptr->p_name;
      myNumber = number;
    }
  else
    fatal("\nProtocol: protocol %u not recognized.\n", number);
}

/********************************************************
 * compare instances of the class.
 * (Polymorphic, called by operator==, !=).
 * PRE: protocol is a Protocol to whom I am being compared.
 * POST: RV == 0 iff my protocol number matches that of protocol,
 *            +X iff my protocol number > that of protocol, and
 *            -X iff my protocol number < that of protocol.
 ********************************************************/

int Protocol::compare(const NetworkObject & netObj) const
{
  const Protocol * ptr = dynamic_cast<const Protocol *>(&netObj);

  if (!ptr)
    fatal("Protocol::compare(): Unable to compare %s and %s",
	  className().c_str(), netObj.className().c_str());

  return myNumber - ptr->number();
}

