/* Client.h defines the template for TCP/IP client objects.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 */

#ifndef CLIENT
#define CLIENT

#include "CommunicatorUser.h"
using namespace std;


template<class T>
class Client : public CommunicatorUser<T>
{
 public:
  Client(const string & service, 
	 const unsigned short port, 
	 const string & remoteHost);
  virtual ~Client();

  virtual void run();
  virtual void interactWithServer() = 0;

  virtual string className() const;

 protected:

 private:

};


/*******************************************************************
 * Constructor.
 * PRE: service is the name of the service for which I am a client
 *       port is the port at which it is offered
 *       remoteHost is where its server is running.
 * POST: I am ready to run.
 ******************************************************************/
template <class T>
inline Client<T>::Client(const string & service, 
			       const unsigned short port, 
			       const string & remoteHost)
{
  CommunicatorUser<T>::setCommunicator(new T(service, port, remoteHost));
}


/******************************************************************
 * Destructor.
 * (nothing to do)
 ******************************************************************/
template<class T>
inline Client<T>::~Client()
{}

/******************************************************************
 * run function defining client behavior.
 * PRE: I am ready to perform my task as a client.
 * POST: I have performed my task as a client.
 * Note: Overload in derived class if different behavior is needed.
 ******************************************************************/
template<class T>
inline void Client<T>::run()
{
   // Optional: replace this line with server interaction prelude

   interactWithServer();

   // Optional: replace this line with server interaction postlude
}

/*******************************************************
 * className accessor.
 * POST: RV == "Client<T>".
 *******************************************************/
template<class T>
inline string Client<T>::className() const
{
  return "Client<T>";
}

#endif


