/* Client_Winsock_Communicator.h defines sockets for TCP clients.
 *
 * Author: Joel Adams
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef CLIENT_WINDOWS_COMMUNICATOR
#define CLIENT_WINDOWS_COMMUNICATOR

#include "Winsock_Communicator.h"
#include "Host.h"
#include "ClientSocketAddress.h"
using namespace std;

class Client_Winsock_Communicator : public Winsock_Communicator
{
 public:
  Client_Winsock_Communicator();
  virtual ~Client_Winsock_Communicator();
  Client_Winsock_Communicator(const Client_Winsock_Communicator &);

  void open(Service * service, const Host & host);
  void close();

  Host & hostInfo();
  virtual string className() const;
  Client_Winsock_Communicator &operator=(const Client_Winsock_Communicator &);

 protected:
  virtual void connect();

 private:
  Host myRemoteHost;
};

/*****************************************************************
 * Copy Construtcor
 * PRE: comm is an object which inherits from Client_Winsock_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline Client_Winsock_Communicator::Client_Winsock_Communicator(const Client_Winsock_Communicator & comm)
    : Winsock_Communicator(comm)
{
    this->myRemoteHost = comm.myRemoteHost;
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits from Client_Winsock_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline Client_Winsock_Communicator &Client_Winsock_Communicator::operator=(const Client_Winsock_Communicator & comm)
{
    Winsock_Communicator::operator=(comm);
    this->myRemoteHost = comm.myRemoteHost;
    return *this;
}


/*****************************************************************
 * Default constructor (nothing to do).
 *****************************************************************/

inline Client_Winsock_Communicator::Client_Winsock_Communicator()
  : Winsock_Communicator()
{}

/*****************************************************************
 * close the socket
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/

inline void Client_Winsock_Communicator::close()
{
  Winsock_Communicator::stop();
}


/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/

inline Client_Winsock_Communicator::~Client_Winsock_Communicator()
{
  if (isOn())
    close();
}


/*****************************************************************
 * Host accessor.
 * POST: RV == myRemoteHost.
 *****************************************************************/

inline Host & Client_Winsock_Communicator::hostInfo()
{
  return myRemoteHost;
}


/*********************************************************
 * className accessor
 * POST: RV == Client_Winsock_Communicator.
 *********************************************************/

inline string Client_Winsock_Communicator::className() const
{
  return "Client_Winsock_Communicator";
}

#endif
