/* ConcurrentServer.h defines an Concurrent server behavior template.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 */

#ifndef CONCURRENT_SERVER
#define CONCURRENT_SERVER

#include "Server.h"
using namespace std;

template <class T>
class ConcurrentServer : public Server<T>
{
 public:
  ConcurrentServer(const string & serviceName, const unsigned short port);
  virtual ~ConcurrentServer();

  virtual void spawnChildForService() = 0;
  virtual void run();

  virtual string className() const;

 protected:

 private:

};


/*******************************************************************
 * Constructor.
 * PRE: serviceName is the name of the service I offer
 *       port is the port at which it is offered.
 * POST: I am ready to run.
 ******************************************************************/

template<class T>
inline ConcurrentServer<T>::ConcurrentServer(const string & serviceName, const unsigned short port)
  : Server<T>(serviceName, port)
{}

/******************************************************************
 * Destructor.
 * (nothing to do)
 ******************************************************************/
template<class T>
inline ConcurrentServer<T>::~ConcurrentServer()
{}



/*******************************************************
 * className accessor.
 * POST: RV == "ConcurrentServer<T>".
 *******************************************************/
template<class T>
inline string ConcurrentServer<T>::className() const
{
  return "ConcurrentServer<T>";
}


/******************************************************************
 * Service function (Concurrent service).
 * PRE: I am running and ready to provide my service to a client.
 * POST: I have provided my service to a client.
 * Note: Overload in derived class if different behavior is needed.
 ******************************************************************/
template<class T>
void ConcurrentServer<T>::run()
{
  try
  {
     for(;;) //ever
     {
	  communicatorInfo()->acceptConnection(); // get initial contact

	  spawnChildForService();        // provide service concurrently

	  // children takes care of closing sockets.
	  // acceptConnection() makes the parent "forget" about the
	  // connection.
     }
  }
  catch(...)
  {
     warning("Server shutting down\n");
     return;
  }
}
#endif
