/* PThreadsConcurrentServer.h defines concurrent server behavior under BSD.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 *
 * Modified by: Ryan VanderBijl
 * Date:        June 2000
 */

#ifndef PTHREADS_CONCURRENT_SERVER
#define PTHREADS_CONCURRENT_SERVER

#include "ConcurrentServer.h"
#include "PoolManager.h"
using namespace std;

template <class T, class Thread_T>
class PThreadsConcurrentServer : public ConcurrentServer<T>
{
 public:
  PThreadsConcurrentServer(const string & serviceName,
			   const unsigned short port);
  virtual ~PThreadsConcurrentServer();

  virtual void spawnChildForService();
  void interactWithClient();  

  virtual string className() const;

 protected:

 private:
    PoolManager<Thread_T> myThreads;
};


/*******************************************************************
 * Constructor.
 * PRE: serviceName is the name of the service I offer
 *       port is the port at which it is offered.
 * POST: I am ready to run.
 ******************************************************************/
template<class T, class Thread_T>
inline PThreadsConcurrentServer<T,Thread_T>::PThreadsConcurrentServer(const string & serviceName, const unsigned short port)
  : ConcurrentServer<T>(serviceName, port)
{}

/******************************************************************
 * Destructor.
 * (nothing to do)
 ******************************************************************/
template<class T, class Thread_T>
inline PThreadsConcurrentServer<T,Thread_T>::~PThreadsConcurrentServer()
{}


/*******************************************************
 * className accessor.
 * POST: RV == "PThreadsConcurrentServer<T,Thread_T>".
 *******************************************************/
template<class T, class Thread_T>
inline string PThreadsConcurrentServer<T,Thread_T>::className() const
{
  return "PThreadsConcurrentServer<T,Thread_T>";
}

/******************************************************************
 * Reproduction function 
 * PRE: I am running and ready to provide my service to a client.
 * POST: A new process has been spawned to provide my service.
 * Note: Overload in derived class if different behavior is needed.
 ******************************************************************/
template<class T, class Thread_T>
inline void PThreadsConcurrentServer<T,Thread_T>::spawnChildForService()
{
  Thread_T *thread= myThreads.getItem();

  if (thread) 
    {
      if (thread->isBusy()) 
	  fatal("We received a busy thread, non busy needed");
      thread->setCommunicator(this->communicatorInfo());
      thread->start();
      communicatorInfo()->forgetConnection();
    }
  else 
    {
      warning("MAIN THREAD DID NOT GET A CHILD THREAD.\n"
	      "IGNORING CONNECTION. ");
      communicatorInfo()->closeConnection();
    }
  // parent goes to get a new connection
}


/*************************
 * interactWithClient.
 *   a PThreadConcurrentServer should never have its 
 *   interactWithClient called, therefore we give an error if it is.
 *   We need this here for interface compliance.
 *************************/
template<class T, class Thread_T>
inline void PThreadsConcurrentServer<T,Thread_T>::interactWithClient()
{
    fatal("This function should never be called. "
	  "For a threaded server, the interactWithClient call "
	  "is defined in the thread class, because the thread "
	  "is the object which actually performs the service.");
}
#endif
