/*************************************************************
 * PosixThread.h -- posix threads in a box (er, class) 
 * All syncronization is contained inside. 
 *
 * Author: Ryan VanderBijl
 * Date:   May/June 2000
 */

#ifndef THREAD
#define THREAD

#ifdef WIN32
#include "winthread\pthread.h"
#else
#include <pthread.h>
#endif /* WIN32 */

#include <stdio.h>
#include <fstream>

class PosixThread {
  public:
     PosixThread();
     virtual ~PosixThread();

     pthread_t    threadID() const;
     bool isBusy();  
     bool setBusy(bool new_status);

     virtual void start();
     virtual void run() = 0;

  protected:
     
  private:
     // function which returns a void *, takes a void * as arguements
     static void* _sleeper(void *); 
     void fatal(char *s);

     pthread_mutex_t myAccessorMutex;
     pthread_mutex_t myWaitMutex;
     pthread_cond_t myCond;
     pthread_t myPosixThreadID;
     bool myBusy;
};


/** if we have a fatal error, give it out **/
inline void PosixThread::fatal(char * s) {
     perror(s);
     ::exit(1);
}

/** 
 * access the thread id. This wont change after the object is
 * created, so, only reads, so no sync-ing needed.
 **/
inline pthread_t PosixThread::threadID() const {
   return myPosixThreadID;
}

#endif /*THREAD*/
