/* Server_TLI_Communicator.h defines a Tranporter for TCP/IP servers
 *    using TLI sockets.
 *
 * Author: Joel Adams
 * Date: November 1998
 * Version: 0.9
 *
 * Modified: Ryan VanderBijl
 * Date:     May/June 2000
 */

#ifndef SERVER_TLI_TRANSPORTER
#define SERVER_TLI_TRANSPORTER

#include "TLI_Communicator.h"
#include "Service.h"
#include "ServerSocketAddress.h"
using namespace std;

class Server_TLI_Communicator : public TLI_Communicator
{
 public:
  Server_TLI_Communicator();
  virtual ~Server_TLI_Communicator();
  Server_TLI_Communicator(const Server_TLI_Communicator & comm);

  void open(Service * service);
  void close();

  virtual void acceptConnection() = 0;
  virtual void closeConnection() = 0;
  virtual bool hasConnection() const;

  virtual string className() const;
  Server_TLI_Communicator & 
      operator=(const Server_TLI_Communicator & comm);

  SocketAddress & client();

 protected:
  void bind();
  void setConnected(bool value);
  void setClient(const SocketAddress & client);

  void setQueueLength(int qLen);
  int queueLength();

 private:
  int myQueueLength;  // TLI wants it for both TCP,UDP
  bool connected;
  SocketAddress myClient;
};



/*****************************************************************
 * Default constructor
 * POST: connected == false.
 *****************************************************************/

inline Server_TLI_Communicator::Server_TLI_Communicator()
  : TLI_Communicator()
{
  connected = false;
  myQueueLength = 0; /* default to zero for UDP */
}

/****************************************************************
 * Connection status indicator.
 * POST: RV == true iff I have accepted a connection.
 ****************************************************************/
inline bool Server_TLI_Communicator::hasConnection() const
{
  return connected;
}

/****************************************************************
 * Connection mutator
 * sets the connection status to value
 ****************************************************************/
inline void Server_TLI_Communicator::setConnected(bool value)
{
  connected = value;
}

/****************************************************************
 * Queue Length Mutator
 * sets the queue length to qLen. Only useful before call to
 * bind().
 ****************************************************************/
inline void Server_TLI_Communicator::setQueueLength(int qLen)
{
  myQueueLength = qLen;
}

/****************************************************************
 * Queue Length Accessor
 * POST: RV == myQueueLength
 ****************************************************************/
inline int Server_TLI_Communicator::queueLength()
{
  return myQueueLength;
}


/*****************************************************************
 * close the socket
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/
inline void Server_TLI_Communicator::close()
{
  TLI_Communicator::stop();
}


/*********************************************************
 * className accessor
 * POST: RV == Server_TLI_Communicator.
 *********************************************************/
inline string Server_TLI_Communicator::className() const
{
  return "Server_TLI_Communicator";
}

/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/
inline Server_TLI_Communicator::~Server_TLI_Communicator()
{
  if (isOn())
    close();
}


/******************************************************
 * client accessor.
 *   May be outdated if there is no current connection.
 *   (is set in acceptConnection, but not unset anywhere)
 * POST: RV == myClient
 ******************************************************/
inline SocketAddress & Server_TLI_Communicator::client()
{
  return myClient;
}

/******************************************************
 * setClient -- sets myClient == cleint
 * PRE: sender is the current client
 ******************************************************/
inline void Server_TLI_Communicator::setClient(const SocketAddress & client)
{
  myClient = client;
}
 
#endif
