/* SocketAddress.h implements the NetworkAddress interface using sockets.
 *
 * Author: Joel Adams.
 * Date: December 1998
 * Version: 0.9
 */

/* if we just used SOCKET_ADDRESS, winsock2.h under windows (VC6.0) 
 * would cause compile problems (in winsock2.h).
 */
#ifndef CLISER_SOCKET_ADDRESS
#define CLISER_SOCKET_ADDRESS

#include "Service.h"
#include "NetworkAddress.h"
using namespace std;

class SocketAddress : public NetworkAddress
{
 public:
  SocketAddress();
  SocketAddress(const string & hostName,
		const unsigned short port);
  SocketAddress(const Host & host,
		const unsigned short port);
  SocketAddress(const Host & host,
		const Service & service);
  SocketAddress(const string & hostName,
		const Service & service);

  SocketAddress(const struct sockaddr_in socketAddress);
  SocketAddress(const SocketAddress &);

  virtual ~SocketAddress();

  struct sockaddr_in * addressInfo();
  struct sockaddr * addressInfoOldFormat();
  unsigned size() const;

  virtual string className() const;
  SocketAddress & operator=(const SocketAddress &);

 protected:

 private:
  struct sockaddr_in myAddress;
};



/*****************************************************************
 * Copy Constructor
 * PRE: sock is a SocketAddress object
 * POST: data members are equal to sock's data memebers.
 *****************************************************************/
inline SocketAddress::SocketAddress(const SocketAddress & sock)
  : NetworkAddress(sock)
{
   ::memcpy(&this->myAddress, &sock.myAddress, sizeof(this->myAddress));
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: sock is a SocketAddress object
 * POST: data members are equal to sock's data memebers.
 *****************************************************************/
inline SocketAddress &SocketAddress::operator=(const SocketAddress & sock)
{
   NetworkAddress::operator=(sock);
   ::memcpy(&this->myAddress, &sock.myAddress, sizeof(this->myAddress));
   return *this;
}


inline SocketAddress::SocketAddress()
  : NetworkAddress()
{
  ::memset(&myAddress, 0, sizeof(struct sockaddr_in));
}

inline SocketAddress::SocketAddress(const string & hostName,
				    const unsigned short port)
  : NetworkAddress(Host(hostName), port)
{
  ::memset(&myAddress, 0, sizeof(struct sockaddr_in));
  myAddress.sin_family = AF_INET;
  myAddress.sin_port = htons(port);
}

inline SocketAddress::SocketAddress(const Host & host,
				    const unsigned short port)
  : NetworkAddress(host, port)
{
  ::memset(&myAddress, 0, sizeof(struct sockaddr_in));
  myAddress.sin_family = AF_INET;
  myAddress.sin_port = htons(port);
}

inline SocketAddress::SocketAddress(const Host & host,
				    const Service & service)
  : NetworkAddress(host, service.portHBO())
{
  ::memset(&myAddress, 0, sizeof(struct sockaddr_in));
  myAddress.sin_family = AF_INET;
  myAddress.sin_port = service.portNBO();
}

inline SocketAddress::SocketAddress(const string & hostName,
				    const Service & service)
  : NetworkAddress(Host(hostName), service.portHBO())
{
  ::memset(&myAddress, 0, sizeof(struct sockaddr_in));
  myAddress.sin_family = AF_INET;
  myAddress.sin_port = service.portNBO();
}



inline SocketAddress::SocketAddress(const struct sockaddr_in socketAddress)
{
  setHost(Host(socketAddress.sin_addr));
  setPort(socketAddress.sin_port);
  memcpy(&myAddress, &socketAddress, sizeof(struct sockaddr_in));
}

inline SocketAddress::~SocketAddress()
{}


inline struct sockaddr_in * SocketAddress::addressInfo()
{
  return &myAddress;
}

inline struct sockaddr * SocketAddress::addressInfoOldFormat()
{
  return (sockaddr *) &myAddress;
}

inline unsigned SocketAddress::size() const
{
  return sizeof(struct sockaddr_in);
}
 
inline string SocketAddress::className() const
{
  return "SocketAddress";
}


#endif
