/* TCP_Server_TLI_Communicator.h defines a Communicator for connection-oriented
 *  TCP/IP Servers using TLI sockets.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 *
 * Modified by: Ryan VanderBijl
 * Date:        May/June 2000
 *
 * Credit to: W. Richard Stevens. Parts of this the function
 *     acceptConnection are derived from code he listed in his Unix 
 *     Network Programming vol 1, 2nd Edition.
 */

#ifndef TCP_SERVER_TLI_TRANSPORTER
#define TCP_SERVER_TLI_TRANSPORTER

#include "Server_TLI_Communicator.h"
#include "TCP_Service.h"
#include "TLI_TCP_SendRecv.h"
using namespace std;

class TCP_Server_TLI_Communicator : public Server_TLI_Communicator,
				    public TLI_TCP_SendRecv
{
 public:
  enum { DEFAULT_QUEUE_LENGTH = 5 };

  TCP_Server_TLI_Communicator(const int queueLength = DEFAULT_QUEUE_LENGTH);
  TCP_Server_TLI_Communicator(const string & service, 
			     const unsigned short port,
			     const int queueLength = DEFAULT_QUEUE_LENGTH);
  TCP_Server_TLI_Communicator(const TCP_Server_TLI_Communicator & comm);
  virtual ~TCP_Server_TLI_Communicator();

  void open(const string & service, const unsigned short port);
  void close();

  virtual int send(const string & msg);
  virtual int receive(string & msg);

  virtual void acceptConnection();
  virtual void closeConnection ();
  virtual void forgetConnection();

  virtual string className() const;
  virtual void print(ostream & out) const;
  TCP_Server_TLI_Communicator & 
      operator=(const TCP_Server_TLI_Communicator & comm);
  
  virtual int getDescriptor() const; //overload the communicator

 protected:
 private:
  struct cli {
     int   connfd;
     struct t_call *tcallp;
     struct cli *next;
  } *cli_head, *cli_tail, *cli_store;

  int myConnectionDescriptor;
};


/*********************************************************
 * className accessor
 * POST: RV == TCP_Server_TLI_Communicator.
 *********************************************************/
inline string TCP_Server_TLI_Communicator::className() const
{
  return "TCP_Server_TLI_Communicator";
}


/************************************************************
 * print utility (polymorphic, called by operator<<).
 * PRE: out is an ostream open for writing.
 * POST: my information has been inserted into out.
 ***********************************************************/
inline void TCP_Server_TLI_Communicator::print(ostream & out) const
{
  TLI_Communicator::print(out);
  out << ", connection descriptor: " << getDescriptor();
}



/*************************************************************
 * session descriptor extractor
 * POST: RV == sessionDescriptor
 *************************************************************/
inline int TCP_Server_TLI_Communicator::getDescriptor() const
{
  return (hasConnection()
		? myConnectionDescriptor
		: TLI_Communicator::getDescriptor());
}


/*************************************************************
 * send -- send a message to the client
 *  Could block.
 * PRE: we should have a client connected,
 *      msg should contain the message to be sent.
 * POST: RV == num chars sent, or -1 if there was an error
 *       (t_errno is set in that case)
 *************************************************************/
inline int TCP_Server_TLI_Communicator::send(const string & msg) 
{
  return TLI_TCP_SendRecv::send(getDescriptor(),msg);
}

/*************************************************************
 * receive -- receive a message from the client
 *  Could block.
 * PRE: we should have a client connected,
 *      
 * POST: msg ill contain the message which was received.
 *       RV == num chars received, or -1 if there was an error
 *       (t_errno is set in that case)
 *************************************************************/
inline int TCP_Server_TLI_Communicator::receive(string & msg) 
{
  return TLI_TCP_SendRecv::receive(getDescriptor(),msg);
}

#endif
