/*****************************************************************
 * TLI_UDP_SendRecv.h
 *  This is a "helper" class for TLI UDP classes. This implements
 *  the actual send/recv calls, so that we do not need to define
 *  them multiple times.
 *
 * It should have just had to be a parent class to the
 * UDP_{Client|Se4rver}_TLI_Communicator classes, but I seems
 * the multiple inheritance/polymorphism  code in gcc 2.95.1
 * is a bit broken. Both the actual call to these functions weren't
 * being found, neither were functions like getDescriptor() when
 * called from in here.
 *
 * Currently the classes which inherrit this class call these
 * functions from their send/receive, and pass the correct
 * info needed.
 *
 * Author: Ryan VanderBijl
 * Date:   June 2000
 */

#ifndef TLI_TCP_SEND_RECV
#define TLI_TCP_SEND_RECV

#include <string>
using namespace std;

class TLI_TCP_SendRecv {
public:
protected:
    int send(int descriptor, const string & msg);
    int receive(int descriptor, string & msg);
private:
};

/************
 * send -- send the message msg, on TLI connection d
 * PRE: d is a connected endpoint,
 *      msg contains the message to be sent.
 * POST: the message has been sent to the server, and the value of
 *       t_snd is returned: number of characters receieved by the
 *       server, or  -1 if error (t_errno is set)
 ************/
inline int TLI_TCP_SendRecv::send(int descriptor, const string & msg) 
{
   return std::t_snd(descriptor, (char*)msg.c_str(), msg.length(),0);
}

/************
 * receive -- receive a message msg, on the TLI connection d.
 * PRE: d is a connected endpoint,
 * POST: the message has been received from the server, and the value of
 *       t_rcv is returned: num chars received, or -1 if error 
 *       (t_errno is set)
 *       The message is stored in msg
 ************/
inline int TLI_TCP_SendRecv::receive(int descriptor, string & msg) {
   const int BUF_SIZE = 1024;
   char buf[BUF_SIZE];
   msg = "";
   int total = 0;
   int count, flags;
   do {
       count = std::t_rcv(descriptor, buf, sizeof(buf), &flags);

       // if there is an error, return 
       if (count == -1) return -1;

       // else store in the message.
       buf[count] = '\0';
       msg += buf;
       total += count;
       if (!(flags & T_MORE)) break;
   } while (count >= BUF_SIZE);
   return total;
}


#endif
