/* UDP_Client_BSD_Communicator.h defines the Communicator abstraction
 *  for UDP client sockets under BSD.
 *
 * Author: Joel Adams.
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef UDP_CLIENT_BSD_TRANSPORTER
#define UDP_CLIENT_BSD_TRANSPORTER

#include "Client_BSD_Communicator.h"
#include "UDP_Service.h"
using namespace std;


class UDP_Client_BSD_Communicator : public Client_BSD_Communicator
{
 public:
  UDP_Client_BSD_Communicator();

  UDP_Client_BSD_Communicator(const string & serviceName,
			const string & remoteHostName);
  UDP_Client_BSD_Communicator(const unsigned short port,
			const string & remoteHostName);
  UDP_Client_BSD_Communicator(const string & serviceName, 
			const unsigned short port,
			const string & remoteHostName);
  UDP_Client_BSD_Communicator(const UDP_Client_BSD_Communicator & comm);

  void open(const string & serviceName,
	    const string & remoteHostName);
  void open(const unsigned short port,
	    const string & remoteHostName);
  void open(const string & serviceName, 
	    const unsigned short port,
	    const string & remoteHostName);

  virtual string className() const;
  UDP_Client_BSD_Communicator & 
      operator=(const UDP_Client_BSD_Communicator & comm);

  virtual ~UDP_Client_BSD_Communicator();

 protected:
 private:
};



/*****************************************************************
 * Copy Construtcor
 * PRE: comm is an object which inherits from UDP_Client_BSD_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline UDP_Client_BSD_Communicator::UDP_Client_BSD_Communicator(const UDP_Client_BSD_Communicator & comm)
    : Client_BSD_Communicator(comm)
{
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits from UDP_Client_BSD_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline UDP_Client_BSD_Communicator &UDP_Client_BSD_Communicator::operator=(const UDP_Client_BSD_Communicator & comm)
{
    Client_BSD_Communicator::operator=(comm);
    return *this;
}


/****************************************************
 * Default constructor
 * POST: Inherited data members have default values.
 ****************************************************/

inline UDP_Client_BSD_Communicator::UDP_Client_BSD_Communicator()
  : Client_BSD_Communicator()
{}


/***************************************************
 * open with string-for-service, string-for-host.
 * PRE: service-name is the name of a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to a UDP_Service for serviceName,
 *       myHost == Host(hostName).
 ***************************************************/

inline void UDP_Client_BSD_Communicator::open(const string & serviceName,
					const string & hostName)
{
  Client_BSD_Communicator::open(new UDP_Service(serviceName), hostName);
}



/***************************************************
 * Construct with string-for-service, string-for-host.
 * PRE: service-name is the name of a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to the Service for serviceName,
 *       myHost == Host(hostName).
 ***************************************************/

inline UDP_Client_BSD_Communicator::
UDP_Client_BSD_Communicator(const string & serviceName,
		      const string & hostName)
  : Client_BSD_Communicator()
{
  open(serviceName, hostName);
}


/***************************************************
 * open with port, string-for-host.
 * PRE: port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to the UDP_Service for port,
 *       myHost == Host(hostName).
 ***************************************************/

inline void UDP_Client_BSD_Communicator::open(const unsigned short port,
					const string & hostName)
{
  Client_BSD_Communicator::open(new UDP_Service(port), hostName);
}


/***************************************************
 * Construct with port, string-for-host.
 * PRE: port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a UDP_Service for port,
 *       myHost == Host(hostName).
 ***************************************************/
inline UDP_Client_BSD_Communicator::
UDP_Client_BSD_Communicator(const unsigned short port,
		      const string & hostName)
  : Client_BSD_Communicator()
{
  open(port, hostName);
}


/***************************************************
 * open with string-for-service, port, string-for-host
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: myServiceInfo points to a TCP_Service for serviceName, port,
 *       myHost == Host(hostName).
 ***************************************************/
inline void UDP_Client_BSD_Communicator::open(const string & serviceName, 
					const unsigned short port,
					const string & hostName)
{
  Client_BSD_Communicator::open(new UDP_Service(serviceName, port), 
			  hostName);
}


/***************************************************
 * Construct with string-for-service, port, string-for-host
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 *       hostName is the name of a valid Internet host.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a TCP_Service for serviceName, port,
 *       myHost == Host(hostName).
 ***************************************************/
inline UDP_Client_BSD_Communicator::
UDP_Client_BSD_Communicator(const string & serviceName, 
		      const unsigned short port,
		      const string & hostName)
  : Client_BSD_Communicator()
{
  open(serviceName, port, hostName);
}


/***************************************************
 * Class className accessor
 * POST: RV == "UDP_Client_BSD_Communicator".
 ***************************************************/

inline string UDP_Client_BSD_Communicator::className() const
{
  return "UDP_Client_BSD_Communicator";
}


/***************************************************
 * Destructor (nothing to do)
 ***************************************************/

inline UDP_Client_BSD_Communicator::~UDP_Client_BSD_Communicator()
{}

#endif
