/* UDP_Server_TLI_Communicator.h defines the Communicator abstraction
 *  for UDP server sockets under TLI.
 *
 * Author: Joel Adams.
 * Date: November 1998
 * Version: 0.9
 */

#ifndef UDP_SERVER_TLI_TRANSPORTER
#define UDP_SERVER_TLI_TRANSPORTER

#include "Server_TLI_Communicator.h"
#include "UDP_Service.h"
#include "TLI_UDP_SendRecv.h"
using namespace std;

class UDP_Server_TLI_Communicator : public Server_TLI_Communicator,
				    public TLI_UDP_SendRecv
{
 public:
  UDP_Server_TLI_Communicator();

  UDP_Server_TLI_Communicator(const string & serviceName);
  UDP_Server_TLI_Communicator(const unsigned short port);
  UDP_Server_TLI_Communicator(const string & serviceName, 
			const unsigned short port);
  virtual ~UDP_Server_TLI_Communicator();
  UDP_Server_TLI_Communicator(const UDP_Server_TLI_Communicator & comm);

  void open(const string & serviceName);
  void open(const unsigned short port);
  void open(const string & serviceName, 
	    const unsigned short port);

  virtual void acceptConnection();    // first contact by a client
  string firstMessageFromClient();    // retrieve first contact msg
  virtual void closeConnection();     // solely for interface compliance
  virtual void forgetConnection();    // solely for interface compliance

  virtual int send   (const string & msg);
  virtual int receive(      string & msg);

  virtual string className() const;
  UDP_Server_TLI_Communicator &
      operator=(const UDP_Server_TLI_Communicator & comm);
  
 protected:
 private:
  string myFirstMessageFromClient;
  typedef struct t_unitdata *pud_t;
  pud_t pud;
};


/****************************************************
 * Default constructor
 * POST: Inherited data members have default values.
 ****************************************************/
inline UDP_Server_TLI_Communicator::UDP_Server_TLI_Communicator()
  : Server_TLI_Communicator()
{
}


/***************************************************
 * Construct with string-for-service.
 * PRE: service-name is the name of a valid UDP service.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to the Service for serviceName.
 ***************************************************/
inline UDP_Server_TLI_Communicator::
UDP_Server_TLI_Communicator(const string & serviceName)
  : Server_TLI_Communicator()
{
   open(serviceName);
}


/***************************************************
 * Construct with port.
 * PRE: port is the port associated with a valid UDP service.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a UDP_Service for port.
 ***************************************************/
inline UDP_Server_TLI_Communicator::
UDP_Server_TLI_Communicator(const unsigned short port)
  : Server_TLI_Communicator()
{
   open(port);
}


/***************************************************
 * Construct with string-for-service, port
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a TCP_Service for serviceName, port.
 ***************************************************/
inline UDP_Server_TLI_Communicator::
UDP_Server_TLI_Communicator(const string & serviceName, 
		      const unsigned short port)
  : Server_TLI_Communicator()
{
   open(serviceName, port);
}



/***************************************************
 * Class className accessor
 * POST: RV == "UDP_Server_TLI_Communicator".
 ***************************************************/
inline string UDP_Server_TLI_Communicator::className() const
{
  return "UDP_Server_TLI_Communicator";
}


/*****************************************************
 * Retrieve first message sent by client
 * PRE: I have accepted a message from a client.
 * POST: RV == firstClientMessage.
 *****************************************************/
inline string UDP_Server_TLI_Communicator::firstMessageFromClient()
{
  return myFirstMessageFromClient;
}

/*****************************************************
 * closeConnection from a connection
 * (do nothing for UDP Communicators,
 *  supplied solely for interface simplicity.)
 *****************************************************/
inline void UDP_Server_TLI_Communicator::closeConnection()
{}

/*****************************************************
 * forgetConnection a connection
 * (do nothing for UDP Communicators,
 *  supplied solely for interface simplicity.)
 *****************************************************/
inline void UDP_Server_TLI_Communicator::forgetConnection()
{}

/***************************************************
 * Destructor (nothing to do)
 ***************************************************/
inline UDP_Server_TLI_Communicator::~UDP_Server_TLI_Communicator()
{
    if (pud) t_free((char*)pud, T_UNITDATA);
}

#endif
