/* UnixConcurrentServer.h defines concurrent server behavior under.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 */

#include <signal.h>

#ifndef UNIX_CONCURRENT_SERVER
#define UNIX_CONCURRENT_SERVER

#include "ConcurrentServer.h"
using namespace std;

template <class T>
class UnixConcurrentServer : public ConcurrentServer<T>
{
 public:
  UnixConcurrentServer(const string & serviceName, const unsigned short port);
  virtual ~UnixConcurrentServer();

  virtual void spawnChildForService();

  virtual string className() const;

 protected:

 private:

};


/*******************************************************************
 * Constructor.
 * PRE: serviceName is the name of the service I offer
 *       port is the port at which it is offered.
 * POST: I am ready to run.
 ******************************************************************/

template<class T>
inline UnixConcurrentServer<T>::UnixConcurrentServer(const string & serviceName, const unsigned short port)
  : ConcurrentServer<T>(serviceName, port)
{
  /* So that we dont have to wait on the child processes. 
   * This SHOULD stop zombie processes from continuously being created.
   */
  signal(SIGCHLD, SIG_IGN);
}

/******************************************************************
 * Destructor.
 * (nothing to do)
 ******************************************************************/
template<class T>
inline UnixConcurrentServer<T>::~UnixConcurrentServer()
{}


/*******************************************************
 * className accessor.
 * POST: RV == "UnixConcurrentServer<T>".
 *******************************************************/
template<class T>
inline string UnixConcurrentServer<T>::className() const
{
  return "UnixConcurrentServer<T>";
}

/******************************************************************
 * Reproduction function
 * PRE: I am running and ready to provide my service to a client.
 * POST: A new process has been spawned to provide my service.
 * Note: Overload in derived class if different behavior is needed.
 ******************************************************************/
template<class T>
inline void UnixConcurrentServer<T>::spawnChildForService()
{
  const int CHILD = 0;

  if (std::fork() == CHILD)
    {
      interactWithClient();
      communicatorInfo()->closeConnection();
      communicatorInfo()->close();
      std::exit(1);
    }
  communicatorInfo()->closeConnection();
  // parent does nothing
}

#endif
