#include "UDP_Server_TLI_Communicator.h"

/*****************************************************
 * acceptConnection first message from a client
 * (receive a message for UDP server transporters,
 *  which can be retrieved via firstMessageFromClient().)
 *****************************************************/
void UDP_Server_TLI_Communicator::acceptConnection()
{
  if (receive(myFirstMessageFromClient) < 0) {
     fatal("acceptConnection error: trying to receive: ");
  }
}


/*****************************************************
 * receive: receive a message from a client
 * POST: msg contains the message.
 *       the client's address has been saved.
 *   Returns the 0 if okay, -1 otherwise (t_errno set)
 *****************************************************/
int UDP_Server_TLI_Communicator::receive(string & msg) 
{
    struct sockaddr_in remote_addr;
    int c = TLI_UDP_SendRecv::receive(getDescriptor(), this->pud, msg);
    if (c != -1) {
	memcpy(&remote_addr, this->pud->addr.buf, sizeof(remote_addr));
	setClient((SocketAddress)remote_addr);
    }
    return c;
}

/*****************************************************
 * send -- send a message to the last client
 * PRE: msg contains the message to be sent.
 * POST: the message is sent.
 *   Returns the 0 if okay, -1 otherwise (t_errno set)
 *****************************************************/
int UDP_Server_TLI_Communicator::send(const string & msg) 
{
    return TLI_UDP_SendRecv::send(getDescriptor(), pud, msg);
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits fromUDP_Server_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
UDP_Server_TLI_Communicator & UDP_Server_TLI_Communicator::operator=(const UDP_Server_TLI_Communicator & comm)
{
    Server_TLI_Communicator::operator=(comm);

    if (!this->pud) {
	pud=(pud_t)t_alloc(getDescriptor(),T_UNITDATA,T_ADDR|T_UDATA);
	if (!pud) fatal("couldnt allocate pud:");
    }

    memcpy(&(this->pud->addr),&(comm.pud->addr),sizeof(struct netbuf));

    this->myFirstMessageFromClient = comm.myFirstMessageFromClient;

    return *this;
}

/*****************************************************************
 * Copy Constructor
 * PRE: comm is an object which inherits fromUDP_Server_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
UDP_Server_TLI_Communicator::UDP_Server_TLI_Communicator(const UDP_Server_TLI_Communicator & comm)
   : Server_TLI_Communicator(comm)
{
    if (!pud) {
       pud = (pud_t)t_alloc(getDescriptor(),T_UNITDATA, T_ADDR|T_UDATA);
       if (!pud) fatal("couldnt allocate pud:");
    }
    memcpy(pud, comm.pud, sizeof(struct t_unitdata));

    this->myFirstMessageFromClient = comm.myFirstMessageFromClient;
}

/***************************************************
 * open with string-for-service.
 * PRE: service-name is the name of a valid UDP service.
 * POST: myServiceInfo points to a UDP_Service for serviceName.
 ***************************************************/
void UDP_Server_TLI_Communicator::open(const string & serviceName)
{
   //zero queue length 
   Server_TLI_Communicator::open(new UDP_Service(serviceName));
   pud = (pud_t)t_alloc(getDescriptor(), T_UNITDATA, T_ADDR | T_UDATA);
   if (!pud) fatal("couldn't allocate pud:");
}



/***************************************************
 * open with port.
 * PRE: port is the port associated with a valid UDP service.
 * POST: myServiceInfo points to the UDP_Service for port.
 ***************************************************/
void UDP_Server_TLI_Communicator::open(const unsigned short port)
{
   Server_TLI_Communicator::open(new UDP_Service(port));
   pud = (pud_t)t_alloc(getDescriptor(), T_UNITDATA, T_ADDR | T_UDATA);
   if (!pud) fatal("Bcouldnt allocate pud:");
}


/***************************************************
 * open with string-for-service, port
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 * POST: myServiceInfo points to a TCP_Service for serviceName, port.
 ***************************************************/
void UDP_Server_TLI_Communicator::open(const string &serviceName,
					const unsigned short port)
{
   Server_TLI_Communicator::open(new UDP_Service(serviceName, port));
   pud = (pud_t)t_alloc(getDescriptor(), T_UNITDATA, T_ADDR | T_UDATA);
   if (!pud) fatal("Ccouldnt allocate pud:");
}
