/** GUI.java provides a GUI for the Cliser system.
 *  @author Joel C. Adams, Spring 2000
 *  All rights reserved
 */
package cliser;

import javax.swing.*;          // JPanel, JComboBox, JButton, ...
import java.awt.*;             // FlowLayout, ...
import java.awt.event.*;       // ActionEvent,
import cliser.*;               // GUI, Generator, JavaGenerator, ...

public class GUI extends JFrame implements ActionListener
{
                                                           /** @serial */
 private final String [] LANGUAGES_SUPPORTED = {"C++", "Java"};
                                                           /** @serial */
 private final String [] INTERFACES_SUPPORTED = {"BSD", "TLI", "Winsock"};
                                                           /** @serial */
 private final String [] PROTOCOLS_SUPPORTED = {"TCP", "UDP"};

 /** Constructor
  *  POST: The GUI has been constructed.
  */
 public GUI()
 {
   setTitle("Cliser");

   myServiceNameLabel = new JLabel("The service's name (one word): ",
                                    SwingConstants.RIGHT);
   myServiceNameTextField = new JTextField(12);
// myServiceNameTextField.setToolTipText("Enter a one-word name for your service\n"
//				 + "(e.g., Daytime, Echo, RandomQuotes, etc.)");
   myServiceNameTextField.addActionListener(this);
   myServiceNamePanel = new JPanel( new GridLayout(1,2) );
   myServiceNamePanel.add(myServiceNameLabel);
   myServiceNamePanel.add(myServiceNameTextField);

   myPortLabel = new JLabel("What port will it use? ", SwingConstants.RIGHT);
   myPortTextField = new JTextField(6);
// myPortTextField.setToolTipText("Ports are positive integers (e.g., 13, 9876). "
//			  + "Standard TCP/IP services use standard ports.  "
//			  + "You must be a sysadmin to use a port below 1024.");
   myPortTextField.addActionListener(this);
   myPortPanel = new JPanel( new GridLayout(1,2) );
   myPortPanel.add(myPortLabel);
   myPortPanel.add(myPortTextField);

   myProtocolLabel = new JLabel("What protocol will it use? ", SwingConstants.RIGHT);
   myProtocolComboBox = new JComboBox(PROTOCOLS_SUPPORTED);
// myProtocolComboBox.setToolTipText("UDP uses fast, but unreliable datagrams "
//				     + "(probably fine for LAN services),"
//				     + " TCP uses slower but reliable connections "
//				     + "(use this for WAN services)");
   myProtocolComboBox.addActionListener(this);
   myProtocolPanel = new JPanel( new GridLayout(1,2) );
   myProtocolPanel.add(myProtocolLabel);
   myProtocolPanel.add(myProtocolComboBox);

   myGenerateLabel = new JLabel("Generate code for a: ", SwingConstants.RIGHT);
   myGenerateComboBox = new JComboBox(new String [] {"Client only", 
                                                     "Server only", 
                                                     "Client and server"});
//   myGenerateComboBox.setToolTipText("What do you want me to do?");
   myGenerateComboBox.addActionListener(this);
   myGeneratePanel = new JPanel( new GridLayout(1,2) );
   myGeneratePanel.add(myGenerateLabel);
   myGeneratePanel.add(myGenerateComboBox);

   myThreadingLabel = new JLabel("Build a server that is: ", SwingConstants.RIGHT);
   myThreadingComboBox = new JComboBox( new String []
                                            {"Iterative (Single threaded)",
                                              "Concurrent (Multithreaded)"} );
// myThreadingComboBox.setToolTipText("If a service can be performed quickly "
//			      + "(i.e., a few hundred operations or less), "
//			      + "use an iterative server; "
//			      + " otherwise use a concurrent server");
   myThreadingComboBox.addActionListener(this);
   myThreadingPanel = new JPanel( new GridLayout(1,2) );
   myThreadingPanel.add(myThreadingLabel);
   myThreadingPanel.add(myThreadingComboBox);

	   mySourceLanguageLabel = new JLabel("Generate source code in: ", SwingConstants.RIGHT);
	   mySourceLanguageComboBox = new JComboBox(LANGUAGES_SUPPORTED);
	//   mySourceLanguageComboBox.setToolTipText("We aim to please.");
	   mySourceLanguageComboBox.addActionListener(this);
	   mySourceLanguagePanel = new JPanel( new GridLayout(1,2) );
	   mySourceLanguagePanel.add(mySourceLanguageLabel);
	   mySourceLanguagePanel.add(mySourceLanguageComboBox);

	   myNetworkInterfaceLabel = new JLabel("Use what network interface? ", SwingConstants.RIGHT);
	   myNetworkInterfaceComboBox = new JComboBox(INTERFACES_SUPPORTED);
	// myNetworkInterfaceComboBox.setToolTipText("Use BSD for Berkeley-based Unixes, "
	//				     + " TLI for AT&T System V Unixes, "
	//				     + " Winsock for MS-Windows");
	   myNetworkInterfaceComboBox.addActionListener(this);
	   myNetworkInterfacePanel = new JPanel( new GridLayout(1,2) );
	   myNetworkInterfacePanel.add(myNetworkInterfaceLabel);
	   myNetworkInterfacePanel.add(myNetworkInterfaceComboBox);

	   myConcurrencyLabel = new JLabel("For concurrency, use: ", SwingConstants.RIGHT);
	   myConcurrencyComboBox = new JComboBox( new String []
							{"Multithreading (POSIX threads)",
							 "Multiprocessing (Unix fork())"});
	// myConcurrencyComboBox.setToolTipText("POSIX threads are lightweight processes "
	//					+ "so use them if they are available "
	//					+ "on your system"); 
	   myConcurrencyComboBox.addActionListener(this);
	   myConcurrencyPanel = new JPanel( new GridLayout(1,2) );
	   myConcurrencyPanel.add(myConcurrencyLabel);
	   myConcurrencyPanel.add(myConcurrencyComboBox);

	   myQuitButton = new JButton("Quit");
	// myQuitButton.setToolTipText("Click here to quit Cliser");
	   myQuitButton.addActionListener(this);
	   myGenerateButton = new JButton("Generate");
	// myGenerateButton.setToolTipText("Click here when you are ready "
	//				   + " to generate source code for your system");
	   myGenerateButton.addActionListener(this);

	   JPanel quitPanel = new JPanel( new FlowLayout() );
	   quitPanel.add(myQuitButton);
	   JPanel genPanel = new JPanel( new FlowLayout() );
	   genPanel.add(myGenerateButton);

	   myButtonPanel = new JPanel( new GridLayout(1,4) );
	   myButtonPanel.add(quitPanel);
	   myButtonPanel.add( new JLabel("") );
	   myButtonPanel.add( new JLabel("") );
	   myButtonPanel.add(genPanel);

	   myControlPanel = new JPanel( new GridLayout(9,1) );
	   myControlPanel.add(myServiceNamePanel);
	   myControlPanel.add(myPortPanel);
	   myControlPanel.add(myProtocolPanel);
	   myControlPanel.add(myGeneratePanel);
	   myControlPanel.add(mySourceLanguagePanel);
	   myControlPanel.add(myNetworkInterfacePanel);
	   myControlPanel.add(myThreadingPanel);
	   myControlPanel.add(myConcurrencyPanel);
	   myControlPanel.add(myButtonPanel);

	   setContentPane(myControlPanel);

	   showNetworkInterfaceControls();  
	   hideThreadControls();
	   hideConcurrencyControls();

	   addWindowListener(new WindowAdapter() 
				 {
				   public void windowClosing(WindowEvent e)     
				   { System.exit(0); }  
				 });

	 }

	 private void showNetworkInterfaceControls()
	 {
	   myNetworkInterfacePanel.setVisible(true);
	 }

	 private void hideNetworkInterfaceControls()
	 {
	   myNetworkInterfacePanel.setVisible(false);
	 }

	 private void showThreadControls()
	 {
	   myThreadingPanel.setVisible(true);
	 }

	 private void hideThreadControls()
	 {
	   myThreadingPanel.setVisible(false);
	 }

	 private void showConcurrencyControls()
	 {
	     myConcurrencyPanel.setVisible(true);
	 }
	   
	 private void hideConcurrencyControls()
	 {
	     myConcurrencyPanel.setVisible(false);
	 }
	   
	 /** event-handler method for Cliser GUI.
	  *  @params event, an ActionEvent.
	  *  POST: event has been handled.
	  */
	 public void actionPerformed(ActionEvent event)
	 {
	   Object eventSource = event.getSource();

	   if ( eventSource instanceof JComboBox 
		|| eventSource instanceof JTextField)
	   {
	       processChoices();
	       processTextFields();
	   }
	   else if ( eventSource instanceof JButton)
	     handleButtons( (JButton) eventSource );
	   else
	     JOptionPane.showMessageDialog(this, "Unexpected event source: " + eventSource,
					    "Error...", JOptionPane.ERROR_MESSAGE);
	 }

	 private void processChoices()
	 {
	   processProtocol();
	   processTargets();
	   processSource();
	   processNetworkInterface();
	   processThreading();
	   processConcurrency();
	   updateDisplay();
	 }
	 
	 private void processProtocol()
	 {
	   usersProtocol = (String) myProtocolComboBox.getSelectedItem();
	 }

	 private void processTargets()
	 {
	   usersTargets = (String) myGenerateComboBox.getSelectedItem();
	   if ( usersTargets.equals("Client only") )
	   {
	     buildClient = true;
	     buildServer = false;
	   }
	   else if ( usersTargets.equals("Server only") 
		     || usersTargets.equals("Client and server") )
	   {
	     buildServer = true;   
	     if ( usersTargets.equals("Client and server") )
	       buildClient = true;
	     else
	       buildClient = false;
	   }
	   else
	   {
	     buildClient = buildServer = false;
	     JOptionPane.showMessageDialog(this, "Unexpected target: " 
					   + usersTargets,
					   "Error...", JOptionPane.ERROR_MESSAGE);
	   }
	 }

	 private void processSource()
	 {
	   usersSourceLanguage = (String) mySourceLanguageComboBox.getSelectedItem();
	 }

	 private void processNetworkInterface()
	 {
	   usersNetworkInterface = (String) myNetworkInterfaceComboBox.getSelectedItem();
	 }

	 private void processThreading()
	 {
	   usersThreading = (String) myThreadingComboBox.getSelectedItem();
	   if ( usersThreading.equals("Iterative (Single threaded)") )
	       concurrent = false;
	   else if (usersThreading.equals("Concurrent (Multithreaded)") )
	       concurrent = true;
	   else
	   {
	     concurrent = false;
	     JOptionPane.showMessageDialog(this, "Unexpected threading choice: " 
					   + usersThreading,
					   "Error...", JOptionPane.ERROR_MESSAGE);
	   }
	 }

	 private void processConcurrency()
	 { 
	   usersConcurrency = (String) myConcurrencyComboBox.getSelectedItem();
	   if (usersConcurrency.equals("Multiprocessing (Unix fork())") )
	     usersConcurrency = "Unix";
	   else if (usersConcurrency.equals("Multithreading (POSIX threads)") )
	     usersConcurrency = "POSIX";
	   else
	   {
	     usersConcurrency = "";
	     JOptionPane.showMessageDialog(this, "Unexpected concurrency choice: " 
					   + usersConcurrency,
					   "Error...", JOptionPane.ERROR_MESSAGE);
	   }
	 }

	 private void updateDisplay()
	 {
	     if (buildServer)
	     {
		 showThreadControls();

		 if (concurrent 
		     && (usersNetworkInterface.equals("BSD")
			 || usersNetworkInterface.equals("TLI") ) )
		     showConcurrencyControls();
		 else
		     hideConcurrencyControls();
	     }
	     else
	     {
		 hideThreadControls();
		 hideConcurrencyControls();
	     }

	     if ( usersSourceLanguage.equals("Java") )
	     {
		 hideNetworkInterfaceControls();
		 hideConcurrencyControls();
	     }
	     else if ( usersSourceLanguage.equals("C++") )
		 showNetworkInterfaceControls();
	     else
		 JOptionPane.showMessageDialog(this, "Unexpected language choice: " 
					       + usersSourceLanguage,
					       "Error...", 
					       JOptionPane.ERROR_MESSAGE);
	 }

	 private void processTextFields()
	 {
	    usersServiceName = myServiceNameTextField.getText();
	    String usersPortString = myPortTextField.getText();
	    if (usersPortString.length() > 0)
	    {
	      try 
		 { usersPort = Integer.parseInt(usersPortString); }
	      catch (Exception e)
		 { usersPort = -1; }
	    }
	    else
	      usersPort = -1;
	 }

	 private void handleButtons(JButton eventSource)
	 {
	     if ( eventSource.equals(myGenerateButton) )
	     {
		 processChoices();
		 processTextFields();

		 Generator codeGenerator = null;
		 if ( this.inputsAreValid() )
		     if ( usersSourceLanguage.equals("Java") )
		     {
			 codeGenerator = new JavaGenerator(usersServiceName, 
							   usersPort, 
							   usersProtocol,
							   this);
			 if (buildClient)
			     codeGenerator.generateClient();

			 if (buildServer)
			 {
			     if (concurrent)
				 codeGenerator.generateServer("Concurrent");
			     else
				 codeGenerator.generateServer("Iterative");
			 }

		     }
		     else if ( usersSourceLanguage.equals("C++") )
		     {
			 codeGenerator = new CPlusPlusGenerator(usersServiceName,
								usersPort,
								usersProtocol,
								usersNetworkInterface,
								this);
			 if (buildClient)
			     codeGenerator.generateClient();

			 if (buildServer)
			 {
			     if (concurrent)
				 codeGenerator.generateServer(usersConcurrency);
			     else
				 codeGenerator.generateServer("Iterative");
			 }

			 if (! usersNetworkInterface.equals("Winsock"))
			 {
			     codeGenerator.generateMakefile(
						buildClient,
						buildServer,
						(concurrent ? usersConcurrency 
							    : "Iterative"));
			 }
		     }
		     else
			 JOptionPane.showMessageDialog(this, 
						       "Unsupported language: " 
						       + usersSourceLanguage,
						       "Error...", 
						       JOptionPane.ERROR_MESSAGE);
	     }
	     else if ( eventSource.equals(myQuitButton) )
		 System.exit(0);
	     else
		JOptionPane.showMessageDialog(this, "Unexpected button: " + eventSource,
					       "Error...", JOptionPane.ERROR_MESSAGE);
	 }

	 private boolean inputsAreValid()
	 {
	     processChoices();
	     processTextFields();

	     if (usersServiceName.length() <= 0)
	     {
		 JOptionPane.showMessageDialog(this, 
					       "You must name your service",
					       "Error...", 
					       JOptionPane.ERROR_MESSAGE);
		 return false;
	     }
	     else if (usersPort <= 0)
	     {
		 JOptionPane.showMessageDialog(this, 
					       "A port must be a positive integer",
					       "Error...", 
					       JOptionPane.ERROR_MESSAGE);
		 return false;
	     }
	     else if (buildServer && concurrent && usersProtocol.equals("UDP") )
	     {
		 JOptionPane.showMessageDialog(this, 
					       "Sorry, concurrent UDP servers unsupported",
					       "Error...", 
					       JOptionPane.ERROR_MESSAGE);
		 return false;
	     }
	     else
		 return true;
	 }


	 public static void main(String [] args)
	 {
	   GUI myGUI = new GUI();
	   myGUI.pack();
	   myGUI.setVisible(true);
	 }

                                                           /** @serial */
	 private String     usersServiceName = "",         // whatever
                                                           /** @serial */
			    usersProtocol = "",            // TCP or UDP
                                                           /** @serial */
			    usersTargets = "",             // client, server, or both
                                                           /** @serial */
			    usersNetworkInterface = "",    // BSD, TLI, or Winsock
                                                           /** @serial */
			    usersSourceLanguage = "",      // Java or C++
                                                           /** @serial */
			    usersThreading = "",           // Iterative or Concurrent
                                                           /** @serial */
			    usersConcurrency = "";         // Unix or POSIX
                                                           /** @serial */
	 private int        usersPort = -1;                // port for service
                                                           /** @serial */
	 private boolean    buildClient = false,           // flag for building client
                                                           /** @serial */
			    buildServer = false,           // flag for building server
                                                           /** @serial */
			    concurrent = false;            // flag for concurrent server
							   // Labels:
                                                           /** @serial */
	 private JLabel     myServiceNameLabel,            // service textfield
                                                           /** @serial */
			    myPortLabel,                   // port textfield
                                                           /** @serial */
			    myProtocolLabel,               // protocol combobox
                                                           /** @serial */
			    myGenerateLabel,               // client, server combobox
                                                           /** @serial */
			    myNetworkInterfaceLabel,       // network interface combobox
                                                           /** @serial */
			    myThreadingLabel,              // kind of threading combobox
                                                           /** @serial */
			    myConcurrencyLabel,            // concurrency combobox
                                                           /** @serial */
			    mySourceLanguageLabel;         // source language combobox
							   // TextFields:
                                                           /** @serial */
	 private JTextField myServiceNameTextField,        // name of the service
                                                           /** @serial */
			    myPortTextField;               // port for service server
							   // ComboBoxes:
                                                           /** @serial */
	 private JComboBox  mySourceLanguageComboBox,      // Java, C++, ?
                                                           /** @serial */
			    myProtocolComboBox,            // TCP or UDP
                                                           /** @serial */
			    myGenerateComboBox,            // Client, Server, or both
                                                           /** @serial */
			    myThreadingComboBox,           // Iterative or Concurrent
                                                           /** @serial */
			    myConcurrencyComboBox,         // Unix fork or POSIX thread
                                                           /** @serial */
			    myNetworkInterfaceComboBox;    // BSD, TLI, or Winsock
							   // Panels:
                                                           /** @serial */
	 private JPanel     myServiceNamePanel,            // name of service
                                                           /** @serial */
			    myPortPanel,                   // service port
                                                           /** @serial */
			    myNetworkInterfacePanel,       // network interface
                                                           /** @serial */
			    myProtocolPanel,               // TCP/IP protocol to use
                                                           /** @serial */
			    myGeneratePanel,               // what to generate
                                                           /** @serial */
			    mySourceLanguagePanel,         // language to generate 
                                                           /** @serial */
			    myThreadingPanel,              // iterative or concurrent
                                                           /** @serial */
			    myConcurrencyPanel,            // Unix vs POSIX
                                                           /** @serial */
			    myButtonPanel,                 // buttons
                                                           /** @serial */
			    myControlPanel;                // the whole thing
							   // Buttons:
                                                           /** @serial */
	 private JButton    myGenerateButton,              // do it
                                                           /** @serial */
			    myQuitButton;                  // quit
	}
