/** Server.java
 *  @author Joel C. Adams
 *  @version 1.0.  Copyright 2000.  All rights reserved.
 */

package cliser;

import cliser.Communicator;
import cliser.CommunicatorUser;
import java.net.*;       // ServerSocket

abstract public class Server extends CommunicatorUser
{
   /** explicit communicator
     *  @params service, a String naming my service
     *  @params port, an int specifying the port I will use
     * POST: my instance variables have been initialized:
     *        myService == service && myPort == port 
     *        && myCommunicator has been initialized by a subclass.
     */
    public Server(String service, int port)
    {
	myPort = port;
	myService = service;
	this.setCommunicator( this.initCommunicator() ); // TCP vs. UDP
    }

    /** myPort accessor
     *  @return myPort.
     */
    public final int getPort()
    {
	return myPort;
    }

    /** accessor for my local host
     *  @return the name of my local host
     */
    public final String getHost()
    {
	String result = null;
	try
        {
	    result = InetAddress.getLocalHost().getHostName();
	}
	catch (Exception e)
        {
	    System.err.println(e); System.exit(1);
	}

	return result;
    }

    /** myService accessor
     *  @return myService.
     */
    public final String getService()
    {
	return myService;
    }

    private int          myPort;
    private String       myService;
}
